ErrorKind

Enum ErrorKind 

pub enum ErrorKind {
    BadSurrogate {
        first: u16,
        second: Option<u16>,
        offset: u8,
    },
    BadUtf8ContByte {
        seq_len: u8,
        offset: u8,
        value: u8,
    },
    Read,
    UnexpectedByte {
        token: Option<Token>,
        expect: Expect,
        actual: u8,
    },
    UnexpectedEof(Token),
}
Category of error that can occur while tokenizing a JSON text.

Variants



BadSurrogate

A Unicode escape sequence of the form \uLLLL or \uHHHH\uLLLL within a string token has a bad Unicode surrogate.

Fields

first: u16

The 16-bit number read from the first Unicode escape sequence.

This will always be a valid Unicode surrogate code unit, either a high surrogate or a low surrogate code pair.

second: Option<u16>

The 16-bit number read from Unicode escape sequence that immediately followed the first escape sequence (if there was one).

This may be a Unicode high surrogate code unit, or it may be a valid Unicode code point, but will never be a low surrogate code unit.

offset: u8

Byte offset from the start of the last Unicode escape sequence (first if second is None, otherwise second) where the error was detected.



BadUtf8ContByte

A UTF-8 continuation byte within a string token has an invalid value.

Fields

seq_len: u8

Length of the UTF-8 byte sequence.

offset: u8

Zero-based offset of the invalid byte value within the sequence.

value: u8

Invalid byte value.



Read

The underlying source of JSON text (e.g. a file or stream) reported an error when the lexical analyzer tried to read the next block of JSON text to analyze.



UnexpectedByte

An unexpected byte was encountered in a token.

Fields

token: Option<Token>

Type of token within which the unexpected byte was encountered.

expect: Expect

Character or characters expected.

actual: u8

The unexpected byte actually encountered.



UnexpectedEof(Token)

The JSON text ended abruptly in the middle of an incomplete lexical token.