pub struct Content(/* private fields */);pipe only.Expand description
Text content of a JSON token identified by a PipeAnalyzer.
See the lexical::Content trait, implemented by this struct, for detailed conceptual
documentation.
Memory considerations
A Content value may hold references to one or more Bytes values that were piped into the
PipeAnalyzer. Consequently, holding on to a Content value may prevent the PipeAnalyzer
from dropping Bytes buffers it has finished scanning. This can lead to increased memory usage.
If all Content values produced by a PipeAnalyzer are retained, it will potentially keep all
inputted Bytes buffers alive. This undermines a key value proposition of a streaming analyzer
and, for large enough JSON texts, may lead to out-of-memory conditions. Therefore, it is advised
that you drop Content values once you have finished examining them.
Implementations
Sourceimpl Content
impl Content
Sourcepub fn literal(&self) -> Literal
pub fn literal(&self) -> Literal
Returns the literal content of the token exactly as it appears in the JSON text.
This is an inherent implementation of lexical::Content::literal for convenience, so it
is available even when you don’t have the trait imported. Refer to the trait documentation
for conceptual details.
Sourcepub fn is_escaped(&self) -> bool
pub fn is_escaped(&self) -> bool
Indicates whether the token content contains escape sequences.
This is an inherent implementation of lexical::Content::is_escaped for convenience, so
it is available even when you don’t have the trait imported. Refer to the trait
documentation for conceptual details.
Sourcepub fn unescaped(&self) -> Unescaped<Literal>
pub fn unescaped(&self) -> Unescaped<Literal>
Returns a normalized version of literal with all escape sequences in the JSON text
fully expanded.
This is an inherent implementation of lexical::Content::unescaped for convenience, so
it is available even when you don’t have the trait imported. Refer to the trait
documentation for conceptual details.
Performance considerations
- If this content belongs to a non-string token, or a string token that contains no escape
sequences, does not allocate, and simply returns an
Unescaped::Literalwrapping theLiteralreturned byliteral, which is a reference to the internals of this content. - If this content belongs to a string token containing at least one escape sequence,
allocates a new owned string value containing the unescaped string content and returns it
wrapped in
Unescaped::Expanded.