Module lexical

Module lexical 

Scan JSON text, extracting a stream of tokens (lexical analysis).

Modules

buf
Convert a fixed-size in-memory buffer into a stream of JSON lexical tokens.
state
Simple state machine for lexical analysis of JSON text.

Enums

ErrorKind
Category of error that can occur while tokenizing a JSON text.
Expect
Character or class of characters expected at the next input position of a JSON text.
Token
Type of lexical token in a JSON text, such as begin object {, literal true, or string.

Traits

Analyzer
Lexical analyzer for JSON text.
Content
Text content of a JSON token.
Error
An error encountered during lexical analysis of JSON text.