Scan JSON text, extracting a stream of tokens (lexical analysis).
Modules
- buf
- Convert a fixed-size in-memory buffer into a stream of JSON lexical tokens.
- state
- Simple state machine for lexical analysis of JSON text.
Enums
- Error
Kind - Category of error that can occur while tokenizing a JSON text.
- Expect
- Character or class of characters expected at the next input position of a JSON text.
- Token
- Type of lexical token in a JSON text, such as begin object
{, literaltrue, or string.