pub enum ErrorKind {
BadSurrogate {
first: u16,
second: Option<u16>,
offset: u8,
},
BadUtf8ContByte {
seq_len: u8,
offset: u8,
value: u8,
},
Read,
UnexpectedByte {
token: Option<Token>,
expect: Expect,
actual: u8,
},
UnexpectedEof(Token),
}Expand description
Category of error that can occur while tokenizing a JSON text.
Variants
BadSurrogate
A Unicode escape sequence of the form \uLLLL or \uHHHH\uLLLL within a
string token has a bad Unicode surrogate.
Fields

first: u16The 16-bit number read from the first Unicode escape sequence.
This will always be a valid Unicode surrogate code unit, either a high surrogate or a low surrogate code pair.
BadUtf8ContByte
A UTF-8 continuation byte within a string token has an invalid value.
Fields
Read
The underlying source of JSON text (e.g. a file or stream) reported an error when the lexical analyzer tried to read the next block of JSON text to analyze.
UnexpectedByte
An unexpected byte was encountered in a token.
Fields
UnexpectedEof(Token)
The JSON text ended abruptly in the middle of an incomplete lexical token.