pub enum Expect {
Boundary,
Char(char),
Digit,
DigitDotExpOrBoundary,
DigitExpOrBoundary,
DigitOrBoundary,
DigitOrExpSign,
DotExpOrBoundary,
EscChar,
StrChar,
TokenStartChar,
UnicodeEscHexDigit,
}Expand description
Character or class of characters expected at the next input position of a JSON text.
This enumeration provides detail information for ErrorKind::UnexpectedByte.
Variants
Boundary
Any token boundary character.
One of:
'{'(opening brace, U+007B)'}'(closing brace, U+007D)'['(opening bracket, U+005B)']'(closing bracket, U+005D)':'(colon, U+003A)','(comma, U+002C)' '(space, U+0020)'\t'(horizontal tab, U+0009)'\n'(line feed, U+000A)'\r'(carriage return, U+000D).
Char(char)
A specific character.
Digit
Any decimal digit character, '0'..'9' (U+0030..U+0039).
DigitDotExpOrBoundary
Any decimal digit character (Digit); the dot or period character '.' (U+002E); one of
the two exponent indicator characters ‘E’ (U+0045) or ‘e’ (U+0065); or any token boundary
character (Boundary).
DigitExpOrBoundary
Any decimal digit character (Digit); or one of the two exponent indicator characters ‘E’
(U+0045) or ‘e’ (U+0065); or any token boundary character (Boundary).
DigitOrBoundary
DigitOrExpSign
Any decimal digit character (Digit) or one of the two exponent sign characters '+'
(U+002B) or '-' (U+002D).
DotExpOrBoundary
(U+0045) or ‘e’ (U+0065); or any token boundary character (Boundary).
EscChar
Any character that completes a short-form escape sequence or starts a Unicode escape sequence.
One of:
'"'(double quotation mark, U+0022)'\\'(reverse solidus, U+005C)'/'(solidus, U+002F)'b'(lowercase ‘b’, U+0062)'f'(lowercase ‘f’, U+0066)'n'(lowercase ‘n’, U+006E)'r'(lowercase ‘r’, U+0072)'t'(lowercase ‘t’, U+0074)'u'(lowercase ‘u’, U+0075)
StrChar
Any character that is valid in a JSON string token, the string token termination character
'"' (double quotation mark, U+0022).
This essentially means any valid Unicode character at or above the space ' ' (U+0020).
TokenStartChar
Any character that validly starts a JSON token.
One of:
UnicodeEscHexDigit
Any hexadecimal digit character allowed in a Unicode escape sequence.
One of:
- A decimal digit character (
Digit) - An uppercase letter
'A'..'F'(U+0041..U+0046) - A lowercase letter
'a'..'f'(U+0061..0066)