Available on crate feature
pointer only.Expand description
Evaluate JSON Pointers against a stream of JSON text.
This module provides a streaming implementation of the JSON Pointer specification.
Modules
- state
- Simple state machine for evaluating JSON Pointers against a JSON pattern.
Structs
- Evaluator
- Evaluates JSON Pointers against a stream of JSON text.
- Group
- An immutable set of JSON Pointers that can be efficiently searched for in a JSON stream.
- Pointer
- A JSON Pointer, as defined in RFC 6901.
- RefTokens
- An iterator over the reference tokens of a JSON Pointer.
Enums
- Event
- An event occurring during JSON Pointer evaluation, e.g. “entered
/foo/bar”. - Pointer
Error - A JSON Pointer syntax error detected when creating a
Pointerfrom an input string.